/// <reference types="mongoose/types/aggregate.js" />
/// <reference types="mongoose/types/callback.js" />
/// <reference types="mongoose/types/collection.js" />
/// <reference types="mongoose/types/connection.js" />
/// <reference types="mongoose/types/cursor.js" />
/// <reference types="mongoose/types/document.js" />
/// <reference types="mongoose/types/error.js" />
/// <reference types="mongoose/types/expressions.js" />
/// <reference types="mongoose/types/helpers.js" />
/// <reference types="mongoose/types/middlewares.js" />
/// <reference types="mongoose/types/indexes.js" />
/// <reference types="mongoose/types/models.js" />
/// <reference types="mongoose/types/mongooseoptions.js" />
/// <reference types="mongoose/types/pipelinestage.js" />
/// <reference types="mongoose/types/populate.js" />
/// <reference types="mongoose/types/query.js" />
/// <reference types="mongoose/types/schemaoptions.js" />
/// <reference types="mongoose/types/schematypes.js" />
/// <reference types="mongoose/types/session.js" />
/// <reference types="mongoose/types/types.js" />
/// <reference types="mongoose/types/utility.js" />
/// <reference types="mongoose/types/validation.js" />
/// <reference types="mongoose/types/virtuals.js" />
/// <reference types="mongoose/types/inferschematype.js" />
import { Type, LoggerService } from '@nestjs/common';
import { ProblemDetailTypeUrlResolver } from '@sjfrhafe/nest-problem-details';
import { Response, Request } from 'express';
import { Primitive, Object as Object$1, Value } from 'json-typescript';
export { Object as JsonObject } from 'json-typescript';
import { FilterQuery, SortOrder, HydratedDocument, InferId } from 'mongoose';
import { ApiExtraModels } from '@nestjs/swagger';
import { SchemaObject } from '@nestjs/swagger/dist/interfaces/open-api-spec.interface.js';

declare enum ValueOperator {
    EQUALS = "$eq",
    NOT_EQUALS = "$neq",
    GREATER_THAN = "$gt",
    GREATER_OR_EQUALS = "$gte",
    LOWER_THAN = "$lt",
    LOWER_OR_EQUALS = "$lte",
    START_WITH = "$start",
    END_WITH = "$end",
    REGEX = "$regex",
    IS_NULL = "$null",
    IS_DEFINED = "$def"
}
/**
 * List operators.
 */
declare enum ListOperator {
    /**
     * Operator for matching several possible value of a field
     */
    IN = "$in",
    /**
     * Operator for refusing all field that contains one of the value in the list
     */
    NOT_IN = "$nin"
}
/**
 * Logical operators
 */
declare enum LogicalOperator {
    /**
     * Operator to make an "or" query
     */
    OR = "$or",
    /**
     * Operator to make an "and" query
     */
    AND = "$and"
}
declare const Operators: readonly (ValueOperator | ListOperator | LogicalOperator)[];
type SearchFieldComparison<T extends object> = {
    [field in keyof T]: Partial<Record<ValueOperator, Primitive> & Record<ListOperator, Array<Primitive>>>;
};
type SearchFieldLogical<T extends object> = {
    [field in LogicalOperator]: Array<Partial<SearchFieldComparison<T>>>;
};
type FlattenObject<Source extends object> = {
    [Key in keyof Source]: (x: Source[Key] extends object ? PrependKey<Key, FlattenObject<Source[Key]>> : Record<Key, Source[Key]>) => void;
}[keyof Source] extends (x: infer I) => void ? {
    [K in keyof I]: I[K];
} : never;
type PrependKey<Key extends PropertyKey, Source> = {
    [P in Exclude<keyof Source, symbol> as `${Exclude<Key, symbol>}.${P}`]: Source[P];
};
type DotKeys<Source extends object> = keyof FlattenObject<Source>;
type SearchField<T extends object> = Partial<SearchFieldComparison<FlattenObject<T>> & SearchFieldLogical<FlattenObject<T>>>;
declare enum FilterParserAction {
    THROW = 0,
    REMOVE = 1,
    DO_NOTHING = 2
}

/**
 * Render one document
 *
 * @param id the MongoDB id of the document
 * @param type the value of `resourceType` (or the value of `modelInjectionName`)
 * @param self the URL pathname of the controller
 * @param resource the DTO version of the MongoDB document
 * @return A JSON compatible representation of the document
 */
type RenderOne<Resource extends Object$1> = (id: string, type: string, self: string, resource: Partial<Resource>) => Object$1;
/**
 * Render a paginated list of documents
 *
 * @param type the value of `resourceType` (or the value of `modelInjectionName`)
 * @param self the URL pathname of the controller
 * @param pageData the current information about the page (the size of a page, and the current page number)
 * @param resources list of item of the page. The key of the map is the MongoDB id of the document, the value the DTO version
 * @return A JSON compatible representation of the document collection
 */
type RenderPage<Resource extends Object$1> = (type: string, self: string, count: number, pageData: {
    size: number;
    current: number;
}, resources: Map<string, Partial<Resource>>) => Object$1;
/**
 * Extract the `Creator` resource from the creation request body
 *
 * @param input the JSON receive by the controller
 * @param type the value of `resourceType` (or the value of `modelInjectionName`)
 */
type ParseCreate<Creator extends Object$1> = (input: Object$1, type: string) => Creator | never;
/**
 * Extract the `Updater` resource from the modification request body
 *
 * @param input the JSON receive by the controller
 * @param type the value of `resourceType` (or the value of `modelInjectionName`)
 * @param id the identifier of the document to update
 */
type ParseUpdate<Updater extends Object$1> = (input: Object$1, type: string, id: string) => Updater | never;
type SwaggerExtensionMaker<Input extends Object$1 = Object$1> = (
/**
 * The class of a DTO
 */
attribute: Type<Input>, 
/**
 * The value of `resourceType` (of the configuration), or the value of `modelInjectionName`
 */
resourceType: string) => SwaggerSchemaExtension;
interface OneResponseSwaggerExtension<Dto extends Object$1> extends SwaggerExtensionMaker<Dto> {
}
interface CollectionResponseSwaggerExtension<Dto extends Object$1> extends SwaggerExtensionMaker<Dto> {
}
interface CreateRequestSwaggerExtension<Creator extends Object$1> extends SwaggerExtensionMaker<Creator> {
}
interface UpdateRequestSwaggerExtension<Updater extends Object$1> extends SwaggerExtensionMaker<Updater> {
}
interface SwaggerSchemaExtension {
    /**
     * List of class to inject in the OpenApi definition
     */
    extraModels: Parameters<typeof ApiExtraModels>;
    /**
     * The linked OpenApi schema
     */
    schema: SchemaObject;
}
type Representation<Dto extends Object$1 = any, Creator extends Object$1 = any, Updater extends Object$1 = any> = {
    /**
     * This function is call to render a document
     */
    readonly renderOne?: RenderOne<Dto>;
    /**
     * This function is call to render a paginated list of documents
     */
    readonly renderPage?: RenderPage<Dto>;
    /**
     * Get OpenApi extra information for representing the response of a document
     */
    readonly getOneResponseSwaggerExtension?: OneResponseSwaggerExtension<Dto>;
    /**
     * Get OpenApi extra information for representing the response of a paginated list of document
     */
    readonly getCollectionResponseSwaggerExtension?: CollectionResponseSwaggerExtension<Dto>;
    /**
     * Get OpenApi extra information for representing the creation body request of a document
     */
    readonly getCreateRequestSwaggerExtension?: CreateRequestSwaggerExtension<Creator>;
    /**
     * Get OpenApi extra information for representing the modification body request of a document
     */
    readonly getUpdateRequestSwaggerExtension?: UpdateRequestSwaggerExtension<Updater>;
    /**
     * Transform the body of creation request into the Creator Dto object
     */
    readonly parseCreateRequest?: ParseCreate<Creator>;
    /**
     * Transform the body of modification request into the Updater Dto object
     */
    readonly parseUpdateRequest?: ParseUpdate<Updater>;
    /**
     * Indicate the output MIME type of your representation, it is also use (with the `Accept` header, or `Content-type` header) to determine which representation to use.
     */
    readonly contentType: string;
};

declare const Hal: Representation;

declare const JsonApi: Required<Representation>;

declare function toMongoOperator<Operator = ListOperator | LogicalOperator | ValueOperator, Value = Primitive | Array<Primitive> | SearchFieldComparison<object>>(operator: Operator, value: Value): [Operator, Value];
/**
 * Transform a received filter into a MongoDB query filter
 * @param inputQuery
 */
declare function toMongoFilterQuery<Resource extends object = object>(inputQuery?: SearchField<Resource>): FilterQuery<object>;
/**
 * Transform a list of field name and negate field name into a MongoDB sort parameter
 * @param input
 */
declare function toMongoSort(input: Array<string>): Record<string, SortOrder>;
/**
 * A preconfigured converter that output a MongoDB Entity as is come from the database
 */
declare class OneToOneConverter<Entity extends Object$1 = Object$1> implements EntityConverter<Entity, Entity, Entity, Entity> {
    fromCreator(input: Partial<Entity>): Partial<Entity>;
    fromUpdater(id: string, input: Partial<Entity>): PartialWithId<Entity>;
    toDto(input: Partial<Entity> | HydratedDocument<Entity>): Partial<Entity>;
    fromSearchable(input?: SearchField<Entity> | undefined): FilterQuery<Entity>;
    fromDtoFields(fields?: Array<keyof FlattenObject<Entity>> | undefined): Array<keyof FlattenObject<Entity>> | undefined;
    fromDtoSort(sort?: Array<string> | undefined): Record<string, SortOrder> | undefined;
}
declare abstract class BaseDto<Data extends Object$1 = Object$1> implements Object$1, Data {
    [any: string]: Value;
}

declare function JsonLdFactory(context: string): Representation & Required<Pick<Representation, "renderOne" | "renderPage" | "getOneResponseSwaggerExtension" | "getCollectionResponseSwaggerExtension">>;

declare const SimpleJson: Representation;

/**
 * Decorator of the DELETE /{id} REST action.
 *
 * Contains Swagger information,
 * Route definition
 * @decorator
 */
declare function DeleteOneDecorator(): MethodDecorator;
/**
 * Decorator of the GET /{id} REST action.
 *
 * Contains Swagger information,
 * Route definition
 * @decorator
 */
declare function GetOneDecorator<Dto extends Object$1>(dtoConstructor: Type<Dto> | undefined, resourceType: string, representations: Array<Representation>, projectionFields?: AddExcludeField): MethodDecorator;
/**
 * Decorator of the POST / REST action.
 *
 * Contains Swagger information,
 * Route definition
 * @decorator
 */
declare function CreateOneDecorator<Creator extends Object$1, Dto extends Object$1>(dtoConstructor: Type<Dto> | undefined, creatorConstructor: Type<Creator> | undefined, resourceType: string, representations: Array<Representation>): MethodDecorator;
/**
 * Decorator of the PATCH /{id} REST action.
 *
 * Contains Swagger information,
 * Route definition
 * @decorator
 */
declare function UpdateOneDecorator<Updater extends Object$1, Dto extends Object$1>(dtoConstructor: Type<Dto> | undefined, updaterConstructor: Type<Updater> | undefined, resourceType: string, representations: Array<Representation>, projectionFields?: AddExcludeField): MethodDecorator;
/**
 * Decorator of the GET / REST action.
 *
 * Contains Swagger information,
 * Route definition
 * @decorator
 */
declare function GetListDecorator<Dto extends Object$1>(dtoConstructor: Type<Dto> | undefined, resourceType: string, representations: Array<Representation>, maxPageLimit: number, operators?: Readonly<Array<(typeof Operators)[number]>>, filterFields?: AddExcludeField, sortFields?: AddExcludeField, projectionFields?: AddExcludeField): MethodDecorator;

/**
 * Converter between the Mongoose Entity and the Nest DTO
 */
interface EntityConverter<Resource extends object, Dto extends Object$1 = Object$1, Creator extends Object$1 = Object$1, Updater extends Object$1 = Object$1> {
    /**
     * Transform a document from MongoDB into the DTO you want to display
     * @param input The document from MongoDB
     */
    toDto: (input: HydratedDocument<Resource> | Partial<Resource>) => Partial<Dto>;
    /**
     * Mapping of the filtering provided to a MongoDB FilterQuery
     * @param input
     */
    fromSearchable: (input?: SearchField<Dto>) => FilterQuery<Resource>;
    /**
     * Mapping of field to select (`projection`) in the MongoDB query from the name of field of the DTO
     * @param fields
     */
    fromDtoFields: (fields?: Array<DotKeys<Dto>>) => Array<DotKeys<Resource>> | undefined;
    /**
     * Transform provided sort option to MongoDB sorting
     * @param sort List of field name of the DTO (can be prefixed by `-` to reverse the order).
     */
    fromDtoSort: <Keys extends string = keyof Dto & string>(sort?: Array<`-${Keys}` | Keys>) => Record<string, SortOrder> | undefined;
    /**
     * Transform a creation DTO into a Mongoose entity data
     * @param input
     */
    fromCreator: (input: Partial<Creator>) => Partial<Resource>;
    /**
     * Transform a modification DTO into a Mongoose entity data
     * @param id
     * @param input
     */
    fromUpdater: (id: string, input: Partial<Updater>) => PartialWithId<Resource>;
}
type JsonOf<Type extends Object$1> = Object$1;

type PartialWithId<Resource> = Partial<Resource> & {
    _id: InferId<Resource>;
};
interface MongooseController<Dto extends Object$1, Creator extends Object$1, Updater extends Object$1> {
    /**
     * Get a paginated list of document
     */
    getList?(response: Response, request: Request, filters?: SearchField<Dto>, page?: {
        size: number;
        number: number;
    }, fields?: Array<string>, sort?: Array<string>, accept?: string): Promise<JsonOf<Dto>>;
    /**
     * Get one document by its Id
     */
    getOne?(response: Response, request: Request, id: string, fields?: Array<string>, accept?: string): Promise<JsonOf<Dto> | never>;
    /**
     * Create one document
     */
    createOne?(response: Response, request: Request, body: JsonOf<Creator>, accept?: string, receiveContentType?: string): Promise<JsonOf<Dto> | undefined | never>;
    /**
     * Get one document
     */
    updateOne?(response: Response, request: Request, body: JsonOf<Updater>, id: string, fields?: Array<string>, noContent?: boolean, accept?: string, receiveContentType?: string): Promise<JsonOf<Dto> | never | undefined>;
    /**
     * Delete one document by its Id
     */
    deleteOne?(id: string): Promise<undefined | never>;
}
type AddExcludeField = {
    /**
     * List of field to remove
     * (default: `[]`)
     */
    exclude: Array<string>;
    /**
     * List of field to add
     * (default: `[]`)
     */
    add: Array<string>;
};
type MongooseControllerOptions<Dto extends Object$1, Creator extends Object$1, Updater extends Object$1> = Partial<{
    /**
     * Allow to remove some part of the controller
     */
    disable: Partial<{
        /**
         * if `true`, the listing of document is removed from the controller (default: `false`)
         */
        list: boolean;
        /**
         * if `true`, getting one document from its id is removed from the controller (default: `false`)
         */
        get: boolean;
        /**
         * if `true`, updating one document is removed from the controller (default: `false`)
         */
        update: boolean;
        /**
         * if `true`, creating a document is removed from the controller (default: `false`)
         */
        create: boolean;
        /**
         * if `true`, deleting one document from its id is removed from the controller (default: `false`)
         */
        delete: boolean;
        /**
         * if `true`, the listing and getting one document are removed from the controller, the output of the creation and modification of a document is disabled (default: `false`)
         */
        read: boolean;
        /**
         * if `true`, updating, creating and deleting a document are removed from the controller (default: `false`)
         */
        write: boolean;
    }>;
    /**
     * Allow to change the pagination size
     */
    pageSize: Partial<{
        /**
         * the default page size if none is provided (default: `10`)
         */
        default: number;
        /**
         * the maximum page size allowed. If the value requested by the user is superior, it is set to this value  (default: `200`)
         */
        max: number;
    }>;
    /**
     * The [`ProblemDetail`](https://github.com/sjfrhafe/nest-problem-details?tab=readme-ov-file#customize-auto-type-generation) error resolver (default: `undefined` => Resolve URL to `https://httpstatuses.com/`)
     */
    urlResolver: ProblemDetailTypeUrlResolver;
    /**
     * The logger to use. Used by `ProblemDetail` (default: `undefined` => no log)
     */
    logger: LoggerService;
    /**
     * The name of the resource to display in the outputs. Used for Json:Api, HAL (default: `undefined` => same as the `modelInjectionName` provided to the factory)
     */
    resourceType: string;
    /**
     * The list of document representation standard to use (default: `[instance of JsonApi, instance of Hal]`)
     */
    representations: Array<Representation<Dto, Creator, Updater>>;
    /**
     * Configuration of the list filter
     */
    filter: Partial<{
        /**
         * List of operators to display in the swagger
         * (default: `[ "$eq", "$neq", "$gt", "$gte", "$lt", "$lte", "$start", "$end", "$regex", "$null", "$def", "$in", "$nin", "$or", "$and" ]`)
         */
        operators: typeof Operators;
        /**
         * Define how the filter parser should handle invalid operator or field.
         * (default: `FilterParserAction.THROW`)
         */
        actionOnInvalid: FilterParserAction;
        /**
         * Filter fields options
         */
        fields: Partial<AddExcludeField>;
    }>;
    /**
     * Sort fields options
     */
    sort: Partial<AddExcludeField>;
    /**
     * Projection (display) fields options
     */
    projection: Partial<AddExcludeField>;
}>;
/**
 * Create a controller class to extends
 *
 * @param modelInjectionName The name linked to the schema (same as declared in `MongooseModule.forFeature`)
 * @param converter The instance responsible to convert your DTO into Mongo Entity and vice-versa
 * @param dtoConstructor The class representation of one element of your collection that you want to return (if missing or `undefined`, it's the same as `options.disable.read: true`)
 * @param creatorConstructor The class representation of a new element to add to your collection that you want to receive (if missing or `undefined`, it's the same as `options.disable.create: true`)
 * @param updaterConstructor The class representation of one element to update in your collection that you want to receive (if missing or `undefined`, it's the same as `options.disable.update: true`)
 * @param options Configurations of the controller
 */
declare function MongooseControllerFactory<Resource extends object, Dto extends Object$1 = Object$1, Creator extends Object$1 = Object$1, Updater extends Object$1 = Object$1>(modelInjectionName: string, converter: EntityConverter<Resource, Dto, Creator, Updater>, dtoConstructor?: Type<Dto>, creatorConstructor?: Type<Creator>, updaterConstructor?: Type<Updater>, options?: MongooseControllerOptions<Dto, Creator, Updater>): Type<MongooseController<Dto, Creator, Updater>>;

export { type AddExcludeField, BaseDto, type CollectionResponseSwaggerExtension, CreateOneDecorator, type CreateRequestSwaggerExtension, DeleteOneDecorator, type DotKeys, type EntityConverter, FilterParserAction, type FlattenObject, GetListDecorator, GetOneDecorator, Hal, JsonApi, JsonLdFactory, type JsonOf, ListOperator, LogicalOperator, type MongooseController, MongooseControllerFactory, type MongooseControllerOptions, type OneResponseSwaggerExtension, OneToOneConverter, Operators, type ParseCreate, type ParseUpdate, type PartialWithId, type RenderOne, type RenderPage, type Representation, type SearchField, SimpleJson, type SwaggerSchemaExtension, UpdateOneDecorator, type UpdateRequestSwaggerExtension, ValueOperator, toMongoFilterQuery, toMongoOperator, toMongoSort };
